@@ -3,59 +3,51 @@
 #include "gfx/pal_spec.hpp"
 
 #include <algorithm>
+#include <array>
 #include <charconv>
 #include <errno.h>
 #include <fstream>
 #include <inttypes.h>
-#include <limits.h>
+#include <ios>
 #include <optional>
 #include <stdint.h>
 #include <stdio.h>
-#include <streambuf>
 #include <string.h>
 #include <string>
 #include <string_view>
 #include <utility>
+#include <vector>
 
 #include "diagnostics.hpp"
 #include "helpers.hpp"
 #include "platform.hpp"
-#include "util.hpp" // UpperMap
+#include "util.hpp" // UpperMap, isDigit
 
 #include "gfx/main.hpp"
 #include "gfx/png.hpp"
+#include "gfx/rgba.hpp"
 #include "gfx/warning.hpp"
 
 using namespace std::string_view_literals;
 
 static char const *hexDigits = "0123456789ABCDEFabcdef";
 
-template<typename Str> // Should be std::string or std::string_view
-static void skipWhitespace(Str const &str, size_t &pos) {
+static void skipBlankSpace(std::string_view const &str, size_t &pos) {
 	pos = std::min(str.find_first_not_of(" \t"sv, pos), str.length());
 }
 
-static constexpr uint8_t nibble(char c) {
-	if (c >= 'a') {
-		assume(c <= 'f');
-		return c - 'a' + 10;
-	} else if (c >= 'A') {
-		assume(c <= 'F');
-		return c - 'A' + 10;
-	} else {
-		assume(c >= '0' && c <= '9');
-		return c - '0';
-	}
+static uint8_t toHex(char c1, char c2) {
+	return parseHexDigit(c1) * 16 + parseHexDigit(c2);
 }
 
-static constexpr uint8_t toHex(char c1, char c2) {
-	return nibble(c1) * 16 + nibble(c2);
-}
-
-static constexpr uint8_t singleToHex(char c) {
+static uint8_t singleToHex(char c) {
 	return toHex(c, c);
 }
 
+static uint16_t toWord(uint8_t low, uint8_t high) {
+	return high << 8 | low;
+}
+
 void parseInlinePalSpec(char const * const rawArg) {
 	// List of #rrggbb/#rgb colors (or #none); comma-separated.
 	// Palettes are separated by colons.
@@ -70,13 +62,11 @@ void parseInlinePalSpec(char const * const rawArg) {
 		error("%s", msg); // `format_` and `-Wformat-security` would complain about `error(msg);`
 		fprintf(
 		    stderr,
-		    "In inline palette spec: %s\n"
-		    "                        ",
-		    rawArg
+		    "In inline palette spec: \"%s\"\n%*c",
+		    rawArg,
+		    static_cast<int>(literal_strlen("In inline palette spec: \"") + ofs),
+		    ' '
 		);
-		for (size_t i = ofs; i; --i) {
-			putc(' ', stderr);
-		}
 		for (size_t i = len; i; --i) {
 			putc('^', stderr);
 		}
@@ -122,8 +112,8 @@ void parseInlinePalSpec(char const * const rawArg) {
 			n = pos;
 		}
 
-		// Skip whitespace, if any
-		skipWhitespace(arg, n);
+		// Skip trailing space, if any
+		skipBlankSpace(arg, n);
 
 		// Skip comma/semicolon, or end
 		if (n == arg.length()) {
@@ -136,7 +126,7 @@ void parseInlinePalSpec(char const * const rawArg) {
 			++nbColors;
 
 			// A trailing comma may be followed by a semicolon
-			skipWhitespace(arg, n);
+			skipBlankSpace(arg, n);
 			if (n == arg.length()) {
 				break;
 			} else if (arg[n] != ';' && arg[n] != ':') {
@@ -151,7 +141,7 @@ void parseInlinePalSpec(char const * const rawArg) {
 		case ':':
 		case ';':
 			++n;
-			skipWhitespace(arg, n);
+			skipBlankSpace(arg, n);
 
 			nbColors = 0; // Start a new palette
 			// Avoid creating a spurious empty palette
@@ -176,24 +166,6 @@ void parseInlinePalSpec(char const * const rawArg) {
 	}
 }
 
-template<typename T, typename U>
-static T readBE(U const *bytes) {
-	T val = 0;
-	for (size_t i = 0; i < sizeof(val); ++i) {
-		val = val << 8 | static_cast<uint8_t>(bytes[i]);
-	}
-	return val;
-}
-
-template<typename T, typename U>
-static T readLE(U const *bytes) {
-	T val = 0;
-	for (size_t i = 0; i < sizeof(val); ++i) {
-		val |= static_cast<uint8_t>(bytes[i]) << (i * 8);
-	}
-	return val;
-}
-
 // Appends the first line read from `file` to the end of the provided `buffer`.
 // Returns true if a line was read.
 [[gnu::warn_unused_result]]
@@ -216,14 +188,6 @@ static bool readLine(std::filebuf &file, std::string &buffer) {
 	}
 }
 
-#define requireLine(kind, filename, file, buffer) \
-	do { \
-		if (!readLine(file, buffer)) { \
-			error(kind " palette file \"%s\" is shorter than expected", filename); \
-			return; \
-		} \
-	} while (0)
-
 static void warnExtraColors(
     char const *kind, char const *filename, uint16_t nbColors, uint16_t maxNbColors
 ) {
@@ -238,15 +202,15 @@ static void warnExtraColors(
 }
 
 // Parses the initial part of a string_view, advancing the "read index" as it does
-template<typename U> // Should be uint*_t
-static std::optional<U> parseDec(std::string const &str, size_t &n) {
+template<typename UintT> // Should be uint*_t
+static std::optional<UintT> parseDec(std::string const &str, size_t &n) {
 	uintmax_t value = 0;
 	auto result = std::from_chars(str.data() + n, str.data() + str.length(), value);
 	if (static_cast<bool>(result.ec)) {
 		return std::nullopt;
 	}
 	n = result.ptr - str.data();
-	return std::optional<U>{value};
+	return std::optional<UintT>{value};
 }
 
 static std::optional<Rgba> parseColor(std::string const &str, size_t &n, uint16_t i) {
@@ -255,7 +219,7 @@ static std::optional<Rgba> parseColor(std::string const &str, size_t &n, uint16_
 		error("Failed to parse color #%d (\"%s\"): invalid red component", i + 1, str.c_str());
 		return std::nullopt;
 	}
-	skipWhitespace(str, n);
+	skipBlankSpace(str, n);
 	if (n == str.length()) {
 		error("Failed to parse color #%d (\"%s\"): missing green component", i + 1, str.c_str());
 		return std::nullopt;
@@ -265,7 +229,7 @@ static std::optional<Rgba> parseColor(std::string const &str, size_t &n, uint16_
 		error("Failed to parse color #%d (\"%s\"): invalid green component", i + 1, str.c_str());
 		return std::nullopt;
 	}
-	skipWhitespace(str, n);
+	skipBlankSpace(str, n);
 	if (n == str.length()) {
 		error("Failed to parse color #%d (\"%s\"): missing blue component", i + 1, str.c_str());
 		return std::nullopt;
@@ -282,25 +246,32 @@ static std::optional<Rgba> parseColor(std::string const &str, size_t &n, uint16_
 static void parsePSPFile(char const *filename, std::filebuf &file) {
 	// https://www.selapa.net/swatches/colors/fileformats.php#psp_pal
 
+#define requireLine() \
+	do { \
+		line.clear(); \
+		if (!readLine(file, line)) { \
+			error("PSP palette file \"%s\" is shorter than expected", filename); \
+			return; \
+		} \
+	} while (0)
+
 	std::string line;
 	if (!readLine(file, line) || line != "JASC-PAL") {
 		error("File \"%s\" is not a valid PSP palette file", filename);
 		return;
 	}
 
-	line.clear();
-	requireLine("PSP", filename, file, line);
+	requireLine();
 	if (line != "0100") {
 		error("Unsupported PSP palette file version \"%s\"", line.c_str());
 		return;
 	}
 
-	line.clear();
-	requireLine("PSP", filename, file, line);
+	requireLine();
 	size_t n = 0;
 	std::optional<uint16_t> nbColors = parseDec<uint16_t>(line, n);
 	if (!nbColors || n != line.length()) {
-		error("Invalid \"number of colors\" line in PSP file (%s)", line.c_str());
+		error("Invalid \"number of colors\" line in PSP file (\"%s\")", line.c_str());
 		return;
 	}
 
@@ -312,8 +283,7 @@ static void parsePSPFile(char const *filename, std::filebuf &file) {
 	options.palSpec.clear();
 
 	for (uint16_t i = 0; i < *nbColors; ++i) {
-		line.clear();
-		requireLine("PSP", filename, file, line);
+		requireLine();
 
 		n = 0;
 		std::optional<Rgba> color = parseColor(line, n, i + 1);
@@ -334,6 +304,8 @@ static void parsePSPFile(char const *filename, std::filebuf &file) {
 		}
 		options.palSpec.back()[i % options.nbColorsPerPal] = *color;
 	}
+
+#undef requireLine
 }
 
 static void parseGPLFile(char const *filename, std::filebuf &file) {
@@ -358,7 +330,7 @@ static void parseGPLFile(char const *filename, std::filebuf &file) {
 		}
 
 		size_t n = 0;
-		skipWhitespace(line, n);
+		skipBlankSpace(line, n);
 		// Skip empty lines, or lines that contain just a comment.
 		if (line.length() == n || line[n] == '#') {
 			continue;
@@ -436,7 +408,7 @@ static void parseACTFile(char const *filename, std::filebuf &file) {
 
 	uint16_t nbColors = 256;
 	if (len == 772) {
-		nbColors = readBE<uint16_t>(&buf[768]);
+		nbColors = toWord(buf[769], buf[768]);
 		if (nbColors > 256 || nbColors == 0) {
 			error("Invalid number of colors in ACT file \"%s\" (%" PRIu16 ")", filename, nbColors);
 			return;
@@ -487,7 +459,7 @@ static void parseACOFile(char const *filename, std::filebuf &file) {
 		error("Failed to read ACO file version");
 		return;
 	}
-	if (readBE<uint16_t>(buf) != 1) {
+	if (toWord(buf[1], buf[0]) != 1) {
 		error("File \"%s\" is not a valid ACO v1 file", filename);
 		return;
 	}
@@ -496,7 +468,7 @@ static void parseACOFile(char const *filename, std::filebuf &file) {
 		error("Failed to read number of colors in palette file");
 		return;
 	}
-	uint16_t nbColors = readBE<uint16_t>(buf);
+	uint16_t nbColors = toWord(buf[1], buf[0]);
 
 	if (uint16_t maxNbColors = options.maxNbColors(); nbColors > maxNbColors) {
 		warnExtraColors("ACO", filename, nbColors, maxNbColors);
@@ -516,7 +488,7 @@ static void parseACOFile(char const *filename, std::filebuf &file) {
 		}
 
 		std::optional<Rgba> &color = options.palSpec.back()[i % options.nbColorsPerPal];
-		uint16_t colorType = readBE<uint16_t>(buf);
+		uint16_t colorType = toWord(buf[1], buf[0]);
 		switch (colorType) {
 		case 0: // RGB
 			// Only keep the MSB of the (big-endian) 16-bit values.
@@ -562,10 +534,10 @@ static void parseGBCFile(char const *filename, std::filebuf &file) {
 		}
 
 		options.palSpec.push_back({
-		    Rgba::fromCGBColor(readLE<uint16_t>(&buf[0])),
-		    Rgba::fromCGBColor(readLE<uint16_t>(&buf[2])),
-		    Rgba::fromCGBColor(readLE<uint16_t>(&buf[4])),
-		    Rgba::fromCGBColor(readLE<uint16_t>(&buf[6])),
+		    Rgba::fromCGBColor(toWord(buf[0], buf[1])),
+		    Rgba::fromCGBColor(toWord(buf[2], buf[3])),
+		    Rgba::fromCGBColor(toWord(buf[4], buf[5])),
+		    Rgba::fromCGBColor(toWord(buf[6], buf[7])),
 		});
 	}
 }
@@ -651,7 +623,7 @@ void parseExternalPalSpec(char const *arg) {
 	// Split both parts, error out if malformed
 	char const *ptr = strchr(arg, ':');
 	if (ptr == nullptr) {
-		error("External palette spec must have format `fmt:path` (missing colon)");
+		error("External palette spec must have format \"fmt:path\" (missing colon)");
 		return;
 	}
 	char const *path = ptr + 1;
@@ -692,7 +664,11 @@ void parseDmgPalSpec(char const * const rawArg) {
 		return;
 	}
 
-	options.palSpecDmg = toHex(arg[0], arg[1]);
+	parseDmgPalSpec(toHex(arg[0], arg[1]));
+}
+
+void parseDmgPalSpec(uint8_t palSpecDmg) {
+	options.palSpecDmg = palSpecDmg;
 
 	// Map gray shades to their DMG color indexes for fast lookup by `Rgba::grayIndex`
 	for (uint8_t i = 0; i < 4; ++i) {
@@ -717,7 +693,7 @@ void parseBackgroundPalSpec(char const *arg) {
 	}
 
 	if (arg[0] != '#') {
-		error("Background color specification must be `#rgb`, `#rrggbb`, or `transparent`");
+		error("Background color specification must be \"#rgb\", \"#rrggbb\", or \"transparent\"");
 		return;
 	}
 
