@@ -4,14 +4,17 @@
 
 #include <deque>
 #include <inttypes.h>
+#include <limits.h>
 #include <stdint.h>
+#include <variant>
 #include <vector>
 
-#include "helpers.hpp" // assume, clz, ctz
+#include "diagnostics.hpp"
+#include "helpers.hpp" // assume
 #include "linkdefs.hpp"
 #include "opmath.hpp"
+#include "verbosity.hpp"
 
-#include "link/main.hpp"
 #include "link/section.hpp"
 #include "link/symbol.hpp"
 #include "link/warning.hpp"
@@ -95,7 +98,6 @@ static int32_t computeRPNExpr(Patch const &patch, std::vector<Symbol> const &fil
 
 	while (size > 0) {
 		RPNCommand command = static_cast<RPNCommand>(getRPNByte(expression, size, patch));
-		int32_t value;
 
 		isError = false;
 
@@ -103,6 +105,7 @@ static int32_t computeRPNExpr(Patch const &patch, std::vector<Symbol> const &fil
 		// C++ does not guarantee order of evaluation of operands!
 		// So, if there are two `popRPN` in the same expression, make
 		// sure the operation is commutative.
+		int32_t value;
 		switch (command) {
 		case RPN_ADD:
 			value = popRPN(patch) + popRPN(patch);
@@ -144,7 +147,7 @@ static int32_t computeRPNExpr(Patch const &patch, std::vector<Symbol> const &fil
 			}
 			break;
 		case RPN_NEG:
-			value = -popRPN(patch);
+			value = op_neg(popRPN(patch));
 			break;
 		case RPN_EXP:
 			value = popRPN(patch);
@@ -158,19 +161,17 @@ static int32_t computeRPNExpr(Patch const &patch, std::vector<Symbol> const &fil
 			break;
 
 		case RPN_HIGH:
-			value = (popRPN(patch) >> 8) & 0xFF;
+			value = op_high(popRPN(patch));
 			break;
 		case RPN_LOW:
-			value = popRPN(patch) & 0xFF;
+			value = op_low(popRPN(patch));
 			break;
 
 		case RPN_BITWIDTH:
-			value = popRPN(patch);
-			value = value != 0 ? 32 - clz(static_cast<uint32_t>(value)) : 0;
+			value = op_bitwidth(popRPN(patch));
 			break;
 		case RPN_TZCOUNT:
-			value = popRPN(patch);
-			value = value != 0 ? ctz(static_cast<uint32_t>(value)) : 32;
+			value = op_tzcount(popRPN(patch));
 			break;
 
 		case RPN_OR:
@@ -278,7 +279,7 @@ static int32_t computeRPNExpr(Patch const &patch, std::vector<Symbol> const &fil
 			if (Symbol const *symbol = getSymbol(fileSymbols, value); !symbol) {
 				errorAt(
 				    patch,
-				    "Requested BANK() of symbol \"%s\", which was not found",
+				    "Requested `BANK()` of undefined symbol `%s`",
 				    fileSymbols[value].name.c_str()
 				);
 				isError = true;
@@ -288,7 +289,7 @@ static int32_t computeRPNExpr(Patch const &patch, std::vector<Symbol> const &fil
 			} else {
 				errorAt(
 				    patch,
-				    "Requested BANK() of non-label symbol \"%s\"",
+				    "Requested `BANK()` of non-label symbol `%s`",
 				    fileSymbols[value].name.c_str()
 				);
 				isError = true;
@@ -303,7 +304,7 @@ static int32_t computeRPNExpr(Patch const &patch, std::vector<Symbol> const &fil
 			while (getRPNByte(expression, size, patch)) {}
 
 			if (Section const *sect = sect_GetSection(name); !sect) {
-				errorAt(patch, "Requested BANK() of section \"%s\", which was not found", name);
+				errorAt(patch, "Requested `BANK()` of undefined section \"%s\"", name);
 				isError = true;
 				value = 1;
 			} else {
@@ -328,7 +329,7 @@ static int32_t computeRPNExpr(Patch const &patch, std::vector<Symbol> const &fil
 			while (getRPNByte(expression, size, patch)) {}
 
 			if (Section const *sect = sect_GetSection(name); !sect) {
-				errorAt(patch, "Requested SIZEOF() of section \"%s\", which was not found", name);
+				errorAt(patch, "Requested `SIZEOF()` of undefined section \"%s\"", name);
 				isError = true;
 				value = 1;
 			} else {
@@ -343,7 +344,7 @@ static int32_t computeRPNExpr(Patch const &patch, std::vector<Symbol> const &fil
 			while (getRPNByte(expression, size, patch)) {}
 
 			if (Section const *sect = sect_GetSection(name); !sect) {
-				errorAt(patch, "Requested STARTOF() of section \"%s\", which was not found", name);
+				errorAt(patch, "Requested `STARTOF()` of undefined section \"%s\"", name);
 				isError = true;
 				value = 1;
 			} else {
@@ -356,7 +357,7 @@ static int32_t computeRPNExpr(Patch const &patch, std::vector<Symbol> const &fil
 		case RPN_SIZEOF_SECTTYPE:
 			value = getRPNByte(expression, size, patch);
 			if (value < 0 || value >= SECTTYPE_INVALID) {
-				errorAt(patch, "Requested SIZEOF() an invalid section type");
+				errorAt(patch, "Requested `SIZEOF()` of an invalid section type");
 				isError = true;
 				value = 0;
 			} else {
@@ -367,7 +368,7 @@ static int32_t computeRPNExpr(Patch const &patch, std::vector<Symbol> const &fil
 		case RPN_STARTOF_SECTTYPE:
 			value = getRPNByte(expression, size, patch);
 			if (value < 0 || value >= SECTTYPE_INVALID) {
-				errorAt(patch, "Requested STARTOF() an invalid section type");
+				errorAt(patch, "Requested `STARTOF()` of an invalid section type");
 				isError = true;
 				value = 0;
 			} else {
@@ -377,15 +378,13 @@ static int32_t computeRPNExpr(Patch const &patch, std::vector<Symbol> const &fil
 
 		case RPN_HRAM:
 			value = popRPN(patch);
-			if (value < 0 || (value > 0xFF && value < 0xFF00) || value > 0xFFFF) {
-				firstErrorAt(patch, "Address $%" PRIx32 " for LDH is not in HRAM range", value);
-				value = 0;
-			} else if (value >= 0 && value <= 0xFF) {
-				warningAt(
+			if (value < 0xFF00 || value > 0xFFFF) {
+				firstErrorAt(
 				    patch,
-				    WARNING_OBSOLETE,
-				    "LDH is deprecated with values from $00 to $FF; use $FF00 to $FFFF"
+				    "Address $%" PRIx32 " for `LDH` is not in HRAM range; use `LD` instead",
+				    value
 				);
+				value = 0;
 			}
 			value &= 0xFF;
 			break;
@@ -394,7 +393,9 @@ static int32_t computeRPNExpr(Patch const &patch, std::vector<Symbol> const &fil
 			value = popRPN(patch);
 			// Acceptable values are 0x00, 0x08, 0x10, ..., 0x38
 			if (value & ~0x38) {
-				firstErrorAt(patch, "Value $%" PRIx32 " is not a RST vector", value);
+				firstErrorAt(
+				    patch, "Value $%" PRIx32 " is not a `RST` vector; use `CALL` instead", value
+				);
 				value = 0;
 			}
 			value |= 0xC7;
@@ -434,8 +435,8 @@ static int32_t computeRPNExpr(Patch const &patch, std::vector<Symbol> const &fil
 					isError = true;
 				}
 			} else if (Symbol const *symbol = getSymbol(fileSymbols, value); !symbol) {
-				errorAt(patch, "Unknown symbol \"%s\"", fileSymbols[value].name.c_str());
-				sym_DumpLocalAliasedSymbols(fileSymbols[value].name);
+				errorAt(patch, "Undefined symbol `%s`", fileSymbols[value].name.c_str());
+				sym_TraceLocalAliasedSymbols(fileSymbols[value].name);
 				isError = true;
 			} else if (std::holds_alternative<Label>(symbol->data)) {
 				Label const &label = std::get<Label>(symbol->data);
@@ -462,7 +463,7 @@ Assertion &patch_AddAssertion() {
 }
 
 void patch_CheckAssertions() {
-	verbosePrint("Checking assertions...\n");
+	verbosePrint(VERB_NOTICE, "Checking assertions...\n");
 
 	for (Assertion &assert : assertions) {
 		int32_t value = computeRPNExpr(assert.patch, *assert.fileSymbols);
@@ -503,30 +504,51 @@ void patch_CheckAssertions() {
 	}
 }
 
+static void checkPatchSize(Patch const &patch, int32_t v, uint8_t n) {
+	assume(n != 0);                     // That doesn't make sense
+	assume(n < CHAR_BIT * sizeof(int)); // Otherwise `1 << n` is UB
+
+	if (v < -(1 << n) || v >= 1 << n) {
+		diagnosticAt(
+		    patch,
+		    WARNING_TRUNCATION_1,
+		    "Value $%" PRIx32 "%s is not %u-bit",
+		    v,
+		    v < 0 ? " (may be negative?)" : "",
+		    n
+		);
+	} else if (v < -(1 << (n - 1))) {
+		diagnosticAt(
+		    patch,
+		    WARNING_TRUNCATION_2,
+		    "Value $%" PRIx32 "%s is not %u-bit",
+		    v,
+		    v < 0 ? " (may be negative?)" : "",
+		    n
+		);
+	}
+}
+
 // Applies all of a section's patches to a data section
 static void applyFilePatches(Section &section, Section &dataSection) {
-	verbosePrint("Patching section \"%s\"...\n", section.name.c_str());
+	verbosePrint(VERB_INFO, "Patching section \"%s\"...\n", section.name.c_str());
 	for (Patch &patch : section.patches) {
 		int32_t value = computeRPNExpr(patch, *section.fileSymbols);
 		uint16_t offset = patch.offset + section.offset;
 
-		struct {
-			uint8_t size;
-			int32_t min;
-			int32_t max;
-		} const types[PATCHTYPE_INVALID] = {
-		    {1, -128,      255      }, // PATCHTYPE_BYTE
-		    {2, -32768,    65536    }, // PATCHTYPE_WORD
-		    {4, INT32_MIN, INT32_MAX}, // PATCHTYPE_LONG
-		    {1, 0,         0        }, // PATCHTYPE_JR
+		uint8_t typeSizes[PATCHTYPE_INVALID] = {
+		    1, // PATCHTYPE_BYTE
+		    2, // PATCHTYPE_WORD
+		    4, // PATCHTYPE_LONG
+		    1, // PATCHTYPE_JR
 		};
-		auto const &type = types[patch.type];
+		uint8_t typeSize = typeSizes[patch.type];
 
-		if (dataSection.data.size() < offset + type.size) {
+		if (dataSection.data.size() < offset + typeSize) {
 			errorAt(
 			    patch,
 			    "Patch would write %zu bytes past the end of section \"%s\" (%zu bytes long)",
-			    offset + type.size - dataSection.data.size(),
+			    offset + typeSize - dataSection.data.size(),
 			    dataSection.name.c_str(),
 			    dataSection.data.size()
 			);
@@ -539,25 +561,18 @@ static void applyFilePatches(Section &section, Section &dataSection) {
 			if (jumpOffset < -128 || jumpOffset > 127) {
 				firstErrorAt(
 				    patch,
-				    "JR target must be between -128 and 127 bytes away, not %" PRId16
-				    "; use JP instead",
+				    "`JR` target must be between -128 and 127 bytes away, not %" PRId16
+				    "; use `JP` instead",
 				    jumpOffset
 				);
 			}
 			dataSection.data[offset] = jumpOffset & 0xFF;
 		} else {
 			// Patch a certain number of bytes
-			if (value < type.min || value > type.max) {
-				diagnosticAt(
-				    patch,
-				    WARNING_TRUNCATION,
-				    "Value $%" PRIx32 "%s is not %u-bit",
-				    value,
-				    value < 0 ? " (may be negative?)" : "",
-				    type.size * 8U
-				);
+			if (typeSize < sizeof(int)) {
+				checkPatchSize(patch, value, typeSize * 8);
 			}
-			for (uint8_t i = 0; i < type.size; ++i) {
+			for (uint8_t i = 0; i < typeSize; ++i) {
 				dataSection.data[offset + i] = value & 0xFF;
 				value >>= 8;
 			}
@@ -565,14 +580,14 @@ static void applyFilePatches(Section &section, Section &dataSection) {
 	}
 }
 
-// Applies all of a section's patches, iterating over "components" of unionized sections
+// Applies all of a section's patches, iterating over "pieces" of unionized sections
 static void applyPatches(Section &section) {
-	if (!sect_HasData(section.type)) {
+	if (!sectTypeHasData(section.type)) {
 		return;
 	}
 
-	for (Section *component = &section; component; component = component->nextu.get()) {
-		applyFilePatches(*component, section);
+	for (Section &piece : section.pieces()) {
+		applyFilePatches(piece, section);
 	}
 }
 
